@@ -1,10 +1,9 @@
-use std::collections::BTreeMap;
 use std::fmt::{Debug, Formatter};
 use std::fs;
 use std::hash::{Hash, Hasher};
 use std::path::{Path, PathBuf};
+use std::{collections::BTreeMap, sync::Arc};
 
-use crate::backend::Backend;
 use crate::backend::backend_type::BackendType;
 use crate::backend::external_plugin_cache::ExternalPluginCache;
 use crate::cache::{CacheManager, CacheManagerBuilder};
@@ -16,23 +15,25 @@ use crate::install_context::InstallContext;
 use crate::plugins::Script::{Download, ExecEnv, Install, ParseIdiomaticFile};
 use crate::plugins::asdf_plugin::AsdfPlugin;
 use crate::plugins::mise_plugin_toml::MisePluginToml;
-use crate::plugins::{Plugin, PluginType, Script, ScriptManager};
-use crate::timeout::run_with_timeout;
+use crate::plugins::{PluginType, Script, ScriptManager};
 use crate::toolset::{ToolRequest, ToolVersion, Toolset};
 use crate::ui::progress_report::SingleReport;
+use crate::{backend::Backend, plugins::PluginEnum, timeout};
 use crate::{dirs, env, file};
+use async_trait::async_trait;
 use color_eyre::eyre::{Result, WrapErr, eyre};
 use console::style;
 use heck::ToKebabCase;
 
 /// This represents a plugin installed to ~/.local/share/mise/plugins
 pub struct AsdfBackend {
-    pub ba: BackendArg,
+    pub ba: Arc<BackendArg>,
     pub name: String,
     pub plugin_path: PathBuf,
     pub repo_url: Option<String>,
     pub toml: MisePluginToml,
-    plugin: Box<AsdfPlugin>,
+    plugin: Arc<AsdfPlugin>,
+    plugin_enum: PluginEnum,
     cache: ExternalPluginCache,
     latest_stable_cache: CacheManager<Option<String>>,
     alias_cache: CacheManager<Vec<(String, String)>>,
@@ -49,6 +50,8 @@ impl AsdfBackend {
             toml_path = plugin_path.join("rtx.plugin.toml");
         }
         let toml = MisePluginToml::from_file(&toml_path).unwrap();
+        let plugin = Arc::new(plugin);
+        let plugin_enum = PluginEnum::Asdf(plugin.clone());
         Self {
             cache: ExternalPluginCache::default(),
             latest_stable_cache: CacheManagerBuilder::new(
@@ -69,16 +72,14 @@ impl AsdfBackend {
             .with_fresh_file(plugin_path.join("bin/list-legacy-filenames"))
             .build(),
             plugin_path,
-            plugin: Box::new(plugin),
+            plugin,
+            plugin_enum,
             repo_url: None,
             toml,
             name,
-            ba,
+            ba: Arc::new(ba),
         }
     }
-    pub fn plugin(&self) -> &dyn Plugin {
-        &*self.plugin
-    }
 
     fn fetch_cached_idiomatic_file(&self, idiomatic_file: &Path) -> Result<Option<String>> {
         let fp = self.idiomatic_cache_file_path(idiomatic_file);
@@ -105,12 +106,12 @@ impl AsdfBackend {
         Ok(())
     }
 
-    fn fetch_bin_paths(&self, tv: &ToolVersion) -> Result<Vec<String>> {
+    async fn fetch_bin_paths(&self, tv: &ToolVersion) -> Result<Vec<String>> {
         let list_bin_paths = self.plugin_path.join("bin/list-bin-paths");
         let bin_paths = if matches!(tv.request, ToolRequest::System { .. }) {
             Vec::new()
         } else if list_bin_paths.exists() {
-            let sm = self.script_man_for_tv(tv)?;
+            let sm = self.script_man_for_tv(tv).await?;
             // TODO: find a way to enable this without deadlocking
             // for (t, tv) in ts.list_current_installed_versions(config) {
             //     if t.name == self.name {
@@ -136,9 +137,14 @@ impl AsdfBackend {
         };
         Ok(bin_paths)
     }
-    fn fetch_exec_env(&self, ts: &Toolset, tv: &ToolVersion) -> Result<EnvMap> {
-        let mut sm = self.script_man_for_tv(tv)?;
-        for p in ts.list_paths() {
+    async fn fetch_exec_env(
+        &self,
+        config: &Arc<Config>,
+        ts: &Toolset,
+        tv: &ToolVersion,
+    ) -> Result<EnvMap> {
+        let mut sm = self.script_man_for_tv(tv).await?;
+        for p in ts.list_paths(config).await {
             sm.prepend_path(p);
         }
         let script = sm.get_script_path(&ExecEnv);
@@ -156,8 +162,8 @@ impl AsdfBackend {
         Ok(env)
     }
 
-    fn script_man_for_tv(&self, tv: &ToolVersion) -> Result<ScriptManager> {
-        let config = Config::get();
+    async fn script_man_for_tv(&self, tv: &ToolVersion) -> Result<ScriptManager> {
+        let config = Config::get().await;
         let mut sm = self.plugin.script_man.clone();
         for (key, value) in tv.request.options().opts {
             let k = format!("RTX_TOOL_OPTS__{}", key.to_uppercase());
@@ -187,7 +193,7 @@ impl AsdfBackend {
             _ => &tv.version,
         };
         // add env vars from mise.toml files
-        for (key, value) in config.env()? {
+        for (key, value) in config.env().await? {
             sm = sm.with_env(key, value.clone());
         }
         let install = tv.install_path().to_string_lossy().to_string();
@@ -223,12 +229,13 @@ impl Hash for AsdfBackend {
     }
 }
 
+#[async_trait]
 impl Backend for AsdfBackend {
     fn get_type(&self) -> BackendType {
         BackendType::Asdf
     }
 
-    fn ba(&self) -> &BackendArg {
+    fn ba(&self) -> &Arc<BackendArg> {
         &self.ba
     }
 
@@ -236,15 +243,15 @@ impl Backend for AsdfBackend {
         Some(PluginType::Asdf)
     }
 
-    fn _list_remote_versions(&self) -> Result<Vec<String>> {
+    async fn _list_remote_versions(&self, _config: &Arc<Config>) -> Result<Vec<String>> {
         self.plugin.fetch_remote_versions()
     }
 
-    fn latest_stable_version(&self) -> Result<Option<String>> {
-        run_with_timeout(
-            || {
+    async fn latest_stable_version(&self, config: &Arc<Config>) -> Result<Option<String>> {
+        timeout::run_with_timeout_async(
+            || async {
                 if !self.plugin.has_latest_stable_script() {
-                    return self.latest_version(Some("latest".into()));
+                    return self.latest_version(config, Some("latest".into())).await;
                 }
                 self.latest_stable_cache
                     .get_or_try_init(|| self.plugin.fetch_latest_stable())
@@ -258,6 +265,7 @@ impl Backend for AsdfBackend {
             },
             SETTINGS.fetch_remote_versions_timeout(),
         )
+        .await
     }
 
     fn get_aliases(&self) -> Result<BTreeMap<String, String>> {
@@ -320,14 +328,14 @@ impl Backend for AsdfBackend {
         Ok(idiomatic_version)
     }
 
-    fn plugin(&self) -> Option<&dyn Plugin> {
-        Some(self.plugin())
+    fn plugin(&self) -> Option<&PluginEnum> {
+        Some(&self.plugin_enum)
     }
 
-    fn install_version_(&self, ctx: &InstallContext, tv: ToolVersion) -> Result<ToolVersion> {
-        let mut sm = self.script_man_for_tv(&tv)?;
+    async fn install_version_(&self, ctx: &InstallContext, tv: ToolVersion) -> Result<ToolVersion> {
+        let mut sm = self.script_man_for_tv(&tv).await?;
 
-        for p in ctx.ts.list_paths() {
+        for p in ctx.ts.list_paths(&ctx.config).await {
             sm.prepend_path(p);
         }
 
@@ -344,24 +352,35 @@ impl Backend for AsdfBackend {
         Ok(tv)
     }
 
-    fn uninstall_version_impl(&self, pr: &Box<dyn SingleReport>, tv: &ToolVersion) -> Result<()> {
+    async fn uninstall_version_impl(
+        &self,
+        pr: &Box<dyn SingleReport>,
+        tv: &ToolVersion,
+    ) -> Result<()> {
         if self.plugin_path.join("bin/uninstall").exists() {
-            self.script_man_for_tv(tv)?
+            self.script_man_for_tv(tv)
+                .await?
                 .run_by_line(&Script::Uninstall, pr)?;
         }
         Ok(())
     }
 
-    fn list_bin_paths(&self, tv: &ToolVersion) -> Result<Vec<PathBuf>> {
+    async fn list_bin_paths(&self, tv: &ToolVersion) -> Result<Vec<PathBuf>> {
         Ok(self
             .cache
-            .list_bin_paths(self, tv, || self.fetch_bin_paths(tv))?
+            .list_bin_paths(self, tv, async || self.fetch_bin_paths(tv).await)
+            .await?
             .into_iter()
             .map(|path| tv.install_path().join(path))
             .collect())
     }
 
-    fn exec_env(&self, config: &Config, ts: &Toolset, tv: &ToolVersion) -> eyre::Result<EnvMap> {
+    async fn exec_env(
+        &self,
+        config: &Arc<Config>,
+        ts: &Toolset,
+        tv: &ToolVersion,
+    ) -> eyre::Result<EnvMap> {
         if matches!(tv.request, ToolRequest::System { .. }) {
             return Ok(BTreeMap::new());
         }
@@ -371,7 +390,10 @@ impl Backend for AsdfBackend {
             return Ok(BTreeMap::new());
         }
         self.cache
-            .exec_env(config, self, tv, || self.fetch_exec_env(ts, tv))
+            .exec_env(config, self, tv, async || {
+                self.fetch_exec_env(config, ts, tv).await
+            })
+            .await
     }
 }
 
@@ -390,12 +412,12 @@ impl Debug for AsdfBackend {
 
 #[cfg(test)]
 mod tests {
-    use test_log::test;
 
     use super::*;
 
-    #[test]
-    fn test_debug() {
+    #[tokio::test]
+    async fn test_debug() {
+        let _config = Config::get().await;
         let plugin = AsdfBackend::from_arg("dummy".into());
         assert!(format!("{plugin:?}").starts_with("AsdfPlugin { name: \"dummy\""));
     }
