@@ -2,32 +2,36 @@ use crate::backend::Backend;
 use crate::backend::backend_type::BackendType;
 use crate::cli::args::BackendArg;
 use crate::cmd::CmdLineRunner;
-use crate::config::Settings;
+use crate::config::{Config, Settings};
 use crate::git::{CloneOptions, Git};
 use crate::install_context::InstallContext;
 use crate::toolset::ToolVersion;
 use crate::{dirs, file, github};
+use async_trait::async_trait;
 use eyre::WrapErr;
 use serde::Deserializer;
 use serde::de::{MapAccess, Visitor};
 use serde_derive::Deserialize;
-use std::fmt::{self, Debug};
 use std::path::PathBuf;
+use std::{
+    fmt::{self, Debug},
+    sync::Arc,
+};
 use url::Url;
 use xx::regex;
 
 #[derive(Debug)]
 pub struct SPMBackend {
-    ba: BackendArg,
+    ba: Arc<BackendArg>,
 }
 
-// https://github.com/apple/swift-package-manager
+#[async_trait]
 impl Backend for SPMBackend {
     fn get_type(&self) -> BackendType {
         BackendType::Spm
     }
 
-    fn ba(&self) -> &BackendArg {
+    fn ba(&self) -> &Arc<BackendArg> {
         &self.ba
     }
 
@@ -35,36 +39,44 @@ impl Backend for SPMBackend {
         Ok(vec!["swift"])
     }
 
-    fn _list_remote_versions(&self) -> eyre::Result<Vec<String>> {
+    async fn _list_remote_versions(&self, _config: &Arc<Config>) -> eyre::Result<Vec<String>> {
         let repo = SwiftPackageRepo::new(&self.tool_name())?;
-        Ok(github::list_releases(repo.shorthand.as_str())?
+        Ok(github::list_releases(repo.shorthand.as_str())
+            .await?
             .into_iter()
             .map(|r| r.tag_name)
             .rev()
             .collect())
     }
 
-    fn install_version_(&self, ctx: &InstallContext, tv: ToolVersion) -> eyre::Result<ToolVersion> {
+    async fn install_version_(
+        &self,
+        ctx: &InstallContext,
+        tv: ToolVersion,
+    ) -> eyre::Result<ToolVersion> {
         let settings = Settings::get();
         settings.ensure_experimental("spm backend")?;
 
         let repo = SwiftPackageRepo::new(&self.tool_name())?;
         let revision = if tv.version == "latest" {
-            self.latest_stable_version()?
+            self.latest_stable_version(&ctx.config)
+                .await?
                 .ok_or_else(|| eyre::eyre!("No stable versions found"))?
         } else {
             tv.version.clone()
         };
         let repo_dir = self.clone_package_repo(ctx, &tv, &repo, &revision)?;
 
-        let executables = self.get_executable_names(&repo_dir, &tv)?;
+        let executables = self.get_executable_names(ctx, &repo_dir, &tv).await?;
         if executables.is_empty() {
             return Err(eyre::eyre!("No executables found in the package"));
         }
         let bin_path = tv.install_path().join("bin");
         file::create_dir_all(&bin_path)?;
         for executable in executables {
-            let exe_path = self.build_executable(&executable, &repo_dir, ctx, &tv)?;
+            let exe_path = self
+                .build_executable(&executable, &repo_dir, ctx, &tv)
+                .await?;
             file::make_symlink(&exe_path, &bin_path.join(executable))?;
         }
 
@@ -78,7 +90,7 @@ impl Backend for SPMBackend {
 
 impl SPMBackend {
     pub fn from_arg(ba: BackendArg) -> Self {
-        Self { ba }
+        Self { ba: Arc::new(ba) }
     }
 
     fn clone_package_repo(
@@ -106,8 +118,9 @@ impl SPMBackend {
         Ok(repo.dir)
     }
 
-    fn get_executable_names(
+    async fn get_executable_names(
         &self,
+        ctx: &InstallContext,
         repo_dir: &PathBuf,
         tv: &ToolVersion,
     ) -> Result<Vec<String>, eyre::Error> {
@@ -122,7 +135,7 @@ impl SPMBackend {
             "--cache-path",
             dirs::CACHE.join("spm"),
         )
-        .full_env(self.dependency_env()?)
+        .full_env(self.dependency_env(&ctx.config).await?)
         .read()?;
         let executables = serde_json::from_str::<PackageDescription>(&package_json)
             .wrap_err("Failed to parse package description")?
@@ -135,11 +148,11 @@ impl SPMBackend {
         Ok(executables)
     }
 
-    fn build_executable(
+    async fn build_executable(
         &self,
         executable: &str,
         repo_dir: &PathBuf,
-        ctx: &InstallContext<'_>,
+        ctx: &InstallContext,
         tv: &ToolVersion,
     ) -> Result<PathBuf, eyre::Error> {
         debug!("Building swift package");
@@ -156,7 +169,12 @@ impl SPMBackend {
             .arg("--cache-path")
             .arg(dirs::CACHE.join("spm"))
             .with_pr(&ctx.pr)
-            .prepend_path(self.dependency_toolset()?.list_paths())?
+            .prepend_path(
+                self.dependency_toolset(&ctx.config)
+                    .await?
+                    .list_paths(&ctx.config)
+                    .await,
+            )?
             .execute()?;
 
         let bin_path = cmd!(
@@ -174,7 +192,7 @@ impl SPMBackend {
             dirs::CACHE.join("spm"),
             "--show-bin-path"
         )
-        .full_env(self.dependency_env()?)
+        .full_env(self.dependency_env(&ctx.config).await?)
         .read()?;
         Ok(PathBuf::from(bin_path.trim().to_string()).join(executable))
     }
@@ -220,12 +238,14 @@ impl SwiftPackageRepo {
 
 #[cfg(test)]
 mod tests {
+    use crate::config::Config;
+
     use super::*;
     use pretty_assertions::assert_str_eq;
-    use test_log::test;
 
-    #[test]
-    fn test_spm_repo_init_by_shorthand() {
+    #[tokio::test]
+    async fn test_spm_repo_init_by_shorthand() {
+        let _config = Config::get().await;
         let package_name = "nicklockwood/SwiftFormat";
         let package_repo = SwiftPackageRepo::new(package_name).unwrap();
         assert_str_eq!(
@@ -235,8 +255,9 @@ mod tests {
         assert_str_eq!(package_repo.shorthand, "nicklockwood/SwiftFormat");
     }
 
-    #[test]
-    fn test_spm_repo_init_name() {
+    #[tokio::test]
+    async fn test_spm_repo_init_name() {
+        let _config = Config::get().await;
         assert!(
             SwiftPackageRepo::new("owner/name.swift").is_ok(),
             "name part can contain ."
@@ -255,8 +276,9 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_spm_repo_init_by_url() {
+    #[tokio::test]
+    async fn test_spm_repo_init_by_url() {
+        let _config = Config::get().await;
         let package_name = "https://github.com/nicklockwood/SwiftFormat.git";
         let package_repo = SwiftPackageRepo::new(package_name).unwrap();
         assert_str_eq!(
