@@ -81,6 +81,7 @@ impl CacheManagerBuilder {
         CacheManager {
             cache_file_path,
             cache: Box::new(OnceCell::new()),
+            cache_async: Box::new(tokio::sync::OnceCell::new()),
             fresh_files: self.fresh_files,
             fresh_duration: self.fresh_duration,
         }
@@ -96,6 +97,7 @@ where
     fresh_duration: Option<Duration>,
     fresh_files: Vec<PathBuf>,
     cache: Box<OnceCell<T>>,
+    cache_async: Box<tokio::sync::OnceCell<T>>,
 }
 
 impl<T> CacheManager<T>
@@ -125,6 +127,33 @@ where
         Ok(val)
     }
 
+    pub async fn get_or_try_init_async<F, Fut>(&self, fetch: F) -> Result<&T>
+    where
+        F: FnOnce() -> Fut,
+        Fut: Future<Output = Result<T>>,
+    {
+        let val = self
+            .cache_async
+            .get_or_try_init(|| async {
+                let path = &self.cache_file_path;
+                if self.is_fresh() {
+                    match self.parse() {
+                        Ok(val) => return Ok::<_, color_eyre::Report>(val),
+                        Err(err) => {
+                            warn!("failed to parse cache file: {} {:#}", path.display(), err);
+                        }
+                    }
+                }
+                let val = fetch().await?;
+                if let Err(err) = self.write(&val) {
+                    warn!("failed to write cache file: {} {:#}", path.display(), err);
+                }
+                Ok(val)
+            })
+            .await?;
+        Ok(val)
+    }
+
     fn parse(&self) -> Result<T> {
         let path = &self.cache_file_path;
         trace!("reading {}", display_path(path));
@@ -275,11 +304,14 @@ pub(crate) fn prune(dir: &Path, opts: &PruneOptions) -> Result<PruneResults> {
 
 #[cfg(test)]
 mod tests {
+    use crate::config::Config;
+
     use super::*;
     use pretty_assertions::assert_eq;
 
-    #[test]
-    fn test_cache() {
+    #[tokio::test]
+    async fn test_cache() {
+        let _config = Config::get().await;
         let cache = CacheManagerBuilder::new(dirs::CACHE.join("test-cache")).build();
         cache.clear().unwrap();
         let val = cache.get_or_try_init(|| Ok(1)).unwrap();
