@@ -19,8 +19,8 @@ pub struct AliasSet {
 }
 
 impl AliasSet {
-    pub fn run(self) -> Result<()> {
-        let mut global_config = Config::try_get()?.global_config()?;
+    pub async fn run(self) -> Result<()> {
+        let mut global_config = Config::get().await.global_config()?;
         global_config.set_alias(&self.plugin, &self.alias, &self.value)?;
         global_config.save()
     }
