@@ -21,10 +21,10 @@ pub struct PluginsLsRemote {
 }
 
 impl PluginsLsRemote {
-    pub fn run(self, config: &Config) -> Result<()> {
-        let installed_plugins = install_state::list_plugins()?;
+    pub async fn run(self, config: &Config) -> Result<()> {
+        let installed_plugins = install_state::list_plugins();
 
-        let shorthands = config.get_shorthands().iter().sorted().collect_vec();
+        let shorthands = config.shorthands.iter().sorted().collect_vec();
         let max_plugin_len = shorthands
             .iter()
             .map(|(plugin, _)| measure_text_width(plugin))
