@@ -1,4 +1,4 @@
-use crate::hash;
+use crate::{config, errors::Error, hash};
 use std::collections::BTreeMap;
 use std::fs;
 use std::hash::{DefaultHasher, Hash, Hasher};
@@ -7,16 +7,14 @@ use std::iter::once;
 use std::ops::Deref;
 use std::path::{Path, PathBuf};
 use std::process::Stdio;
-use std::sync::{Arc, Mutex};
+use std::sync::Arc;
 use std::time::{Duration, SystemTime};
-use std::{panic, thread};
 
 use super::args::ToolArg;
 use crate::cli::Cli;
 use crate::cmd::CmdLineRunner;
 use crate::config::{Config, SETTINGS};
 use crate::env_diff::EnvMap;
-use crate::errors::Error;
 use crate::file::display_path;
 use crate::task::task_file_providers::TaskFileProvidersBuilder;
 use crate::task::{Deps, GetMatchingExt, Task};
@@ -27,7 +25,6 @@ use crate::ui::{ctrlc, prompt, style, time};
 use crate::{dirs, env, exit, file, ui};
 use clap::{CommandFactory, ValueHint};
 use console::Term;
-use crossbeam_channel::{select, unbounded};
 use demand::{DemandOption, Select};
 use duct::IntoExecutablePath;
 use eyre::{Result, bail, ensure, eyre};
@@ -36,6 +33,10 @@ use indexmap::IndexMap;
 use itertools::Itertools;
 #[cfg(unix)]
 use nix::sys::signal::SIGTERM;
+use tokio::{
+    sync::{Mutex, Semaphore},
+    task::JoinSet,
+};
 use xx::regex;
 
 /// Run task(s)
@@ -173,7 +174,7 @@ pub struct Run {
     pub is_linear: bool,
 
     #[clap(skip)]
-    pub failed_tasks: Mutex<Vec<(Task, i32)>>,
+    pub failed_tasks: std::sync::Mutex<Vec<(Task, i32)>>,
 
     /// Change how tasks information is output when running tasks
     ///
@@ -191,13 +192,13 @@ pub struct Run {
     pub tmpdir: PathBuf,
 
     #[clap(skip)]
-    pub keep_order_output: Mutex<IndexMap<Task, KeepOrderOutputs>>,
+    pub keep_order_output: std::sync::Mutex<IndexMap<Task, KeepOrderOutputs>>,
 
     #[clap(skip)]
     pub task_prs: IndexMap<Task, Arc<Box<dyn SingleReport>>>,
 
     #[clap(skip)]
-    pub timed_outputs: Arc<Mutex<IndexMap<String, (SystemTime, String)>>>,
+    pub timed_outputs: Arc<std::sync::Mutex<IndexMap<String, (SystemTime, String)>>>,
 
     // Do not use cache on remote tasks
     #[clap(long, verbatim_doc_comment, env = "MISE_TASK_REMOTE_NO_CACHE")]
@@ -207,7 +208,8 @@ pub struct Run {
 type KeepOrderOutputs = (Vec<(String, String)>, Vec<(String, String)>);
 
 impl Run {
-    pub fn run(mut self) -> Result<()> {
+    pub async fn run(mut self) -> Result<()> {
+        let config = Config::try_get().await?;
         if self.task == "-h" {
             self.get_clap_command().print_help()?;
             return Ok(());
@@ -223,9 +225,9 @@ impl Run {
             .chain(self.args.clone())
             .chain(self.args_last.clone())
             .collect_vec();
-        let task_list = get_task_lists(&args, true)?;
+        let task_list = get_task_lists(&config, &args, true).await?;
         time!("run get_task_lists");
-        self.parallelize_tasks(task_list)?;
+        self.parallelize_tasks(task_list).await?;
         time!("run done");
         Ok(())
     }
@@ -238,14 +240,15 @@ impl Run {
             .clone()
     }
 
-    fn parallelize_tasks(mut self, mut tasks: Vec<Task>) -> Result<()> {
-        time!("paralellize_tasks start");
+    async fn parallelize_tasks(mut self, tasks: Vec<Task>) -> Result<()> {
+        time!("parallelize_tasks start");
 
         ctrlc::exit_on_ctrl_c(false);
 
         if self.output(None) == TaskOutput::Timed {
             let timed_outputs = self.timed_outputs.clone();
-            thread::spawn(move || {
+            tokio::spawn(async move {
+                let mut interval = tokio::time::interval(Duration::from_millis(100));
                 loop {
                     {
                         let mut outputs = timed_outputs.lock().unwrap();
@@ -261,13 +264,15 @@ impl Run {
                             }
                         }
                     }
-                    thread::sleep(Duration::from_millis(100));
+                    interval.tick().await;
                 }
             });
         }
 
-        self.fetch_tasks(&mut tasks)?;
-        let tasks = Deps::new(tasks)?;
+        let mut tasks = resolve_depends(tasks).await?;
+        self.fetch_tasks(&mut tasks).await?;
+
+        let tasks = Deps::new(tasks).await?;
         for task in tasks.all() {
             self.validate_task(task)?;
             match self.output(Some(task)) {
@@ -288,105 +293,87 @@ impl Run {
         let num_tasks = tasks.all().count();
         self.is_linear = tasks.is_linear();
         self.output = Some(self.output(None));
+        let this = Arc::new(self);
 
-        let mut all_tools = self.tool.clone();
+        let mut all_tools = this.tool.clone();
         for t in tasks.all() {
             for (k, v) in &t.tools {
                 all_tools.push(format!("{k}@{v}").parse()?);
             }
         }
+        let config = Config::get().await;
         let mut ts = ToolsetBuilder::new()
             .with_args(&all_tools)
-            .build(&Config::get())?;
+            .build(&config)
+            .await?;
 
-        ts.install_missing_versions(&InstallOptions {
-            missing_args_only: !SETTINGS.task_run_auto_install,
-            ..Default::default()
-        })?;
+        ts.install_missing_versions(
+            &config,
+            &InstallOptions {
+                missing_args_only: !SETTINGS.task_run_auto_install,
+                ..Default::default()
+            },
+        )
+        .await?;
 
-        let tasks = Mutex::new(tasks);
         let timer = std::time::Instant::now();
+        let this_ = this.clone();
+        let jset = Arc::new(Mutex::new(JoinSet::new()));
+        let jset_ = jset.clone();
 
-        let pool = rayon::ThreadPoolBuilder::new()
-            .num_threads(self.jobs() + 1)
-            .build()?;
-        pool.scope(|s| {
-            let (tx_err, rx_err) = unbounded();
-            let run = |task: &Task| {
-                let t = task.clone();
-                let tx_err = tx_err.clone();
-                s.spawn(|_| {
-                    let task = t;
-                    let tx_err = tx_err;
-                    let prefix = task.estyled_prefix();
-                    panic::set_hook(Box::new(move |info| {
-                        prefix_eprintln!(prefix, "panic in task: {info}");
-                        exit(1);
-                    }));
-                    if !self.is_stopping() {
-                        trace!("running task: {task}");
-                        if let Err(err) = self.run_task(&task) {
-                            let status = Error::get_exit_status(&err);
-                            if !self.is_stopping() && status.is_none() {
-                                // only show this if it's the first failure, or we haven't killed all the remaining tasks
-                                // otherwise we'll get unhelpful error messages about being killed by mise which we expect
-                                let prefix = task.estyled_prefix();
-                                let err_body = if SETTINGS.verbose {
-                                    format!("{} {err:?}", style::ered("ERROR"))
-                                } else {
-                                    format!("{} {err}", style::ered("ERROR"))
-                                };
-                                self.eprint(&task, &prefix, &err_body);
-                            }
-                            let _ = tx_err.send((task.clone(), status));
-                        }
-                    }
-                    tasks.lock().unwrap().remove(&task);
-                });
-            };
-            let rx = tasks.lock().unwrap().subscribe();
-            while !self.is_stopping() {
-                select! {
-                    recv(rx) -> task => { // receive a task from Deps
-                        if let Some(task) = task.unwrap() {
-                            run(&task);
-                        }else {
-                            // we get the None value which means the channel closes
-                            break;
-                        }
-                    }
-                    recv(rx_err) -> task => { // a task errored
-                        let (task, status) = task.unwrap();
-                        self.add_failed_task(task, status);
-                        if !self.continue_on_error {
-                            #[cfg(unix)]
-                            CmdLineRunner::kill_all(SIGTERM); // start killing other running tasks
-                            #[cfg(windows)]
-                            CmdLineRunner::kill_all();
-                        }
-                    }
+        let handle = tokio::task::spawn(async move {
+            let tasks = Arc::new(Mutex::new(tasks));
+            let semaphore = Arc::new(Semaphore::new(this_.jobs()));
+            let mut rx = tasks.lock().await.subscribe();
+            while let Some(Some(task)) = rx.recv().await {
+                if this_.is_stopping() {
+                    break;
                 }
+                trace!("running task: {task}");
+                let jset = jset_.clone();
+                let this_ = this_.clone();
+                let semaphore = semaphore.clone();
+                let tasks = tasks.clone();
+                let config = config.clone();
+                jset.lock().await.spawn(async move {
+                    let _permit = semaphore.acquire().await?;
+                    let result = this_.run_task(&task, &config).await;
+                    if let Err(err) = &result {
+                        let status = Error::get_exit_status(err);
+                        if !this_.is_stopping() && status.is_none() {
+                            // only show this if it's the first failure, or we haven't killed all the remaining tasks
+                            // otherwise we'll get unhelpful error messages about being killed by mise which we expect
+                            let prefix = task.estyled_prefix();
+                            let err_body = if SETTINGS.verbose {
+                                format!("{} {err:?}", style::ered("ERROR"))
+                            } else {
+                                format!("{} {err}", style::ered("ERROR"))
+                            };
+                            this_.eprint(&task, &prefix, &err_body);
+                        }
+                        this_.add_failed_task(task.clone(), status);
+                    }
+                    tasks.lock().await.remove(&task);
+                    result
+                });
             }
         });
 
-        if let Some((task, status)) = self.failed_tasks.lock().unwrap().first() {
-            let prefix = task.estyled_prefix();
-            self.eprint(
-                task,
-                &prefix,
-                &format!("{} task failed", style::ered("ERROR")),
-            );
-            exit(*status);
+        while let Some(result) = jset.lock().await.join_next().await {
+            if result.is_ok() || this.continue_on_error {
+                continue;
+            }
+            #[cfg(unix)]
+            CmdLineRunner::kill_all(SIGTERM);
+            #[cfg(windows)]
+            CmdLineRunner::kill_all();
+            break;
         }
+        handle.await?;
 
-        if self.timings() && num_tasks > 1 && *env::MISE_TASK_LEVEL == 0 {
-            let msg = format!("Finished in {}", time::format_duration(timer.elapsed()));
-            eprintln!("{}", style::edim(msg));
-        };
-
-        if self.output(None) == TaskOutput::KeepOrder {
+        if this.output(None) == TaskOutput::KeepOrder {
             // TODO: display these as tasks complete in order somehow rather than waiting until everything is done
-            let output = self.keep_order_output.lock().unwrap();
+            let output = this.keep_order_output.lock().unwrap();
             for (out, err) in output.values() {
                 for (prefix, line) in out {
                     if console::colors_enabled() {
@@ -404,6 +391,19 @@ impl Run {
                 }
             }
         }
+        if this.timings() && num_tasks > 1 && *env::MISE_TASK_LEVEL == 0 {
+            let msg = format!("Finished in {}", time::format_duration(timer.elapsed()));
+            eprintln!("{}", style::edim(msg));
+        };
+        if let Some((task, status)) = this.failed_tasks.lock().unwrap().first() {
+            let prefix = task.estyled_prefix();
+            this.eprint(
+                task,
+                &prefix,
+                &format!("{} task failed", style::ered("ERROR")),
+            );
+            exit(*status);
+        }
         time!("parallelize_tasks done");
 
         Ok(())
@@ -421,7 +421,7 @@ impl Run {
         }
     }
 
-    fn run_task(&self, task: &Task) -> Result<()> {
+    async fn run_task(&self, task: &Task, config: &Arc<Config>) -> Result<()> {
         let prefix = task.estyled_prefix();
         if SETTINGS.task_skip.contains(&task.name) {
             if !self.quiet(Some(task)) {
@@ -429,7 +429,7 @@ impl Run {
             }
             return Ok(());
         }
-        if !self.force && self.sources_are_fresh(task)? {
+        if !self.force && self.sources_are_fresh(task, config).await? {
             if !self.quiet(Some(task)) {
                 self.eprint(task, &prefix, "sources up-to-date, skipping");
             }
@@ -441,13 +441,16 @@ impl Run {
             }
         }
 
-        let config = Config::get();
+        let config = Config::get().await;
         let mut tools = self.tool.clone();
         for (k, v) in &task.tools {
             tools.push(format!("{k}@{v}").parse()?);
         }
-        let ts = ToolsetBuilder::new().with_args(&tools).build(&config)?;
-        let mut env = task.render_env(&ts)?;
+        let ts = ToolsetBuilder::new()
+            .with_args(&tools)
+            .build(&config)
+            .await?;
+        let mut env = task.render_env(&config, &ts).await?;
         let output = self.output(Some(task));
         env.insert("MISE_TASK_OUTPUT".into(), output.to_string());
         if output == TaskOutput::Prefix {
@@ -477,10 +480,11 @@ impl Run {
         let timer = std::time::Instant::now();
 
         if let Some(file) = &task.file {
-            self.exec_file(file, task, &env, &prefix)?;
+            self.exec_file(&config, file, task, &env, &prefix).await?;
         } else {
-            let rendered_run_scripts =
-                task.render_run_scripts_with_args(self.cd.clone(), &task.args, &env)?;
+            let rendered_run_scripts = task
+                .render_run_scripts_with_args(&config, self.cd.clone(), &task.args, &env)
+                .await?;
 
             let get_args = || {
                 [String::new()]
@@ -489,14 +493,16 @@ impl Run {
                     .cloned()
                     .collect()
             };
-            self.parse_usage_spec_and_init_env(task, &mut env, get_args)?;
+            self.parse_usage_spec_and_init_env(&config, task, &mut env, get_args)
+                .await?;
 
             for (script, args) in rendered_run_scripts {
-                self.exec_script(&script, &args, task, &env, &prefix)?;
+                self.exec_script(&script, &args, task, &env, &prefix)
+                    .await?;
             }
         }
 
-        if self.timings() && (task.file.as_ref().is_some() || !task.run().is_empty()) {
+        if self.task_timings() && (task.file.as_ref().is_some() || !task.run().is_empty()) {
             self.eprint(
                 task,
                 &prefix,
@@ -509,7 +515,7 @@ impl Run {
         Ok(())
     }
 
-    fn exec_script(
+    async fn exec_script(
         &self,
         script: &str,
         args: &[String],
@@ -517,13 +523,13 @@ impl Run {
         env: &BTreeMap<String, String>,
         prefix: &str,
     ) -> Result<()> {
-        let config = Config::get();
+        let config = Config::get().await;
         let script = script.trim_start();
-        let cmd = style::ebold(format!("$ {script} {args}", args = args.join(" ")))
-            .bright()
-            .to_string();
+        let cmd = format!("$ {script} {args}", args = args.join(" ")).to_string();
         if !self.quiet(Some(task)) {
-            let msg = trunc(prefix, config.redact(cmd).trim());
+            let msg = style::ebold(trunc(prefix, config.redact(cmd).trim()))
+                .bright()
+                .to_string();
             self.eprint(task, prefix, &msg)
         }
 
@@ -535,10 +541,10 @@ impl Run {
             tmp.flush()?;
             drop(tmp);
             file::make_executable(&file)?;
-            self.exec(&file, args, task, env, prefix)
+            self.exec(&file, args, task, env, prefix).await
         } else {
             let (program, args) = self.get_cmd_program_and_args(script, task, args)?;
-            self.exec_program(&program, &args, task, env, prefix)
+            self.exec_program(&program, &args, task, env, prefix).await
         }
     }
 
@@ -601,13 +607,20 @@ impl Run {
         }
     }
 
-    fn exec_file(&self, file: &Path, task: &Task, env: &EnvMap, prefix: &str) -> Result<()> {
-        let config = Config::get();
+    async fn exec_file(
+        &self,
+        config: &Arc<Config>,
+        file: &Path,
+        task: &Task,
+        env: &EnvMap,
+        prefix: &str,
+    ) -> Result<()> {
         let mut env = env.clone();
         let command = file.to_string_lossy().to_string();
         let args = task.args.iter().cloned().collect_vec();
         let get_args = || once(command.clone()).chain(args.clone()).collect_vec();
-        self.parse_usage_spec_and_init_env(task, &mut env, get_args)?;
+        self.parse_usage_spec_and_init_env(config, task, &mut env, get_args)
+            .await?;
 
         if !self.quiet(Some(task)) {
             let cmd = format!("{} {}", display_path(file), args.join(" "))
@@ -618,10 +631,10 @@ impl Run {
             self.eprint(task, prefix, &cmd);
         }
 
-        self.exec(file, &args, task, &env, prefix)
+        self.exec(file, &args, task, &env, prefix).await
     }
 
-    fn exec(
+    async fn exec(
         &self,
         file: &Path,
         args: &[String],
@@ -630,10 +643,10 @@ impl Run {
         prefix: &str,
     ) -> Result<()> {
         let (program, args) = self.get_file_program_and_args(file, task, args)?;
-        self.exec_program(&program, &args, task, env, prefix)
+        self.exec_program(&program, &args, task, env, prefix).await
     }
 
-    fn exec_program(
+    async fn exec_program(
         &self,
         program: &str,
         args: &[String],
@@ -641,7 +654,7 @@ impl Run {
         env: &BTreeMap<String, String>,
         prefix: &str,
     ) -> Result<()> {
-        let config = Config::get();
+        let config = Config::get().await;
         let program = program.to_executable();
         let redactions = config.redactions();
         let raw = self.raw(Some(task));
@@ -728,7 +741,7 @@ impl Run {
                 }
             }
         }
-        let dir = self.cwd(task)?;
+        let dir = self.cwd(task, &config).await?;
         if !dir.exists() {
             self.eprint(
                 task,
@@ -811,13 +824,14 @@ impl Run {
         Ok(())
     }
 
-    fn parse_usage_spec_and_init_env(
+    async fn parse_usage_spec_and_init_env(
         &self,
+        config: &Arc<Config>,
         task: &Task,
         env: &mut EnvMap,
         get_args: impl Fn() -> Vec<String>,
     ) -> Result<()> {
-        let (spec, _) = task.parse_usage_spec(self.cd.clone(), env)?;
+        let (spec, _) = task.parse_usage_spec(config, self.cd.clone(), env).await?;
         if !spec.cmd.args.is_empty() || !spec.cmd.flags.is_empty() {
             let args: Vec<String> = get_args();
             debug!("Parsing usage spec for {:?}", args);
@@ -833,15 +847,15 @@ impl Run {
         Ok(())
     }
 
-    fn sources_are_fresh(&self, task: &Task) -> Result<bool> {
+    async fn sources_are_fresh(&self, task: &Task, config: &Arc<Config>) -> Result<bool> {
         let outputs = task.outputs.paths(task);
         if task.sources.is_empty() && outputs.is_empty() {
             return Ok(false);
         }
         // TODO: We should benchmark this and find out if it might be possible to do some caching around this or something
         // perhaps using some manifest in a state directory or something, maybe leveraging atime?
-        let run = || -> Result<bool> {
-            let root = self.cwd(task)?;
+        let run = async || -> Result<bool> {
+            let root = self.cwd(task, config).await?;
             let mut sources = task.sources.clone();
             sources.push(task.config_source.to_string_lossy().to_string());
             let source_metadatas = self.get_file_metadatas(&root, &sources)?;
@@ -870,7 +884,7 @@ impl Run {
                 _ => Ok(false),
             }
         };
-        Ok(run().unwrap_or_else(|err| {
+        Ok(run().await.unwrap_or_else(|err| {
             warn!("sources_are_fresh: {err:?}");
             false
         }))
@@ -975,13 +989,13 @@ impl Run {
         Ok(last_mod)
     }
 
-    fn cwd(&self, task: &Task) -> Result<PathBuf> {
+    async fn cwd(&self, task: &Task, config: &Arc<Config>) -> Result<PathBuf> {
         if let Some(d) = &self.cd {
             Ok(d.clone())
-        } else if let Some(d) = task.dir()? {
+        } else if let Some(d) = task.dir(config).await? {
             Ok(d)
         } else {
-            Ok(Config::get()
+            Ok(config
                 .project_root
                 .clone()
                 .or_else(|| dirs::CWD.clone())
@@ -1003,14 +1017,19 @@ impl Run {
     }
 
     fn timings(&self) -> bool {
-        !self.quiet(None)
-            && !self.no_timings
-            && SETTINGS
-                .task_timings
-                .unwrap_or(self.output == Some(TaskOutput::Prefix))
+        !self.quiet(None) && !self.no_timings
     }
 
-    fn fetch_tasks(&self, tasks: &mut Vec<Task>) -> Result<()> {
+    fn task_timings(&self) -> bool {
+        self.timings()
+            && SETTINGS.task_timings.unwrap_or(
+                self.output == Some(TaskOutput::Prefix)
+                    || self.output == Some(TaskOutput::Timed)
+                    || self.output == Some(TaskOutput::KeepOrder),
+            )
+    }
+
+    async fn fetch_tasks(&self, tasks: &mut Vec<Task>) -> Result<()> {
         let no_cache = self.no_cache || SETTINGS.task_remote_no_cache.unwrap_or(false);
         let task_file_providers = TaskFileProvidersBuilder::new()
             .with_cache(!no_cache)
@@ -1026,7 +1045,7 @@ impl Run {
                     bail!("No provider found for file: {}", source);
                 }
 
-                let local_path = provider.unwrap().get_local_path(&source).unwrap();
+                let local_path = provider.unwrap().get_local_path(&source).await?;
 
                 t.file = Some(local_path);
             }
@@ -1156,15 +1175,15 @@ fn trunc(prefix: &str, msg: &str) -> String {
     console::truncate_str(msg, *env::TERM_WIDTH - prefix_len - 1, "…").to_string()
 }
 
-fn err_no_task(name: &str) -> Result<()> {
-    if Config::get().tasks().is_ok_and(|t| t.is_empty()) {
+async fn err_no_task(config: &Config, name: &str) -> Result<()> {
+    if config.tasks().await.is_ok_and(|t| t.is_empty()) {
         bail!(
             "no tasks defined in {}. Are you in a project directory?",
             display_path(dirs::CWD.clone().unwrap_or_default())
         );
     }
     if let Some(cwd) = &*dirs::CWD {
-        let includes = Config::get().task_includes_for_dir(cwd);
+        let includes = config::task_includes_for_dir(cwd, &config.config_files);
         let path = includes
             .iter()
             .map(|d| d.join(name))
@@ -1189,9 +1208,9 @@ fn err_no_task(name: &str) -> Result<()> {
     bail!("no task {} found", style::ered(name));
 }
 
-fn prompt_for_task() -> Result<Task> {
-    let config = Config::get();
-    let tasks = config.tasks()?;
+async fn prompt_for_task() -> Result<Task> {
+    let config = Config::get().await;
+    let tasks = config.tasks().await?;
     ensure!(
         !tasks.is_empty(),
         "no tasks defined. see {url}",
@@ -1204,7 +1223,7 @@ fn prompt_for_task() -> Result<Task> {
     for t in tasks.values().filter(|t| !t.hide) {
         s = s.option(
             DemandOption::new(&t.name)
-                .label(&t.display_name())
+                .label(&t.display_name)
                 .description(&t.description),
         );
     }
@@ -1221,8 +1240,13 @@ fn prompt_for_task() -> Result<Task> {
     }
 }
 
-pub fn get_task_lists(args: &[String], prompt: bool) -> Result<Vec<Task>> {
-    args.iter()
+pub async fn get_task_lists(
+    config: &Arc<Config>,
+    args: &[String],
+    prompt: bool,
+) -> Result<Vec<Task>> {
+    let args = args
+        .iter()
         .map(|s| vec![s.to_string()])
         .coalesce(|a, b| {
             if b == vec![":::".to_string()] {
@@ -1234,45 +1258,59 @@ pub fn get_task_lists(args: &[String], prompt: bool) -> Result<Vec<Task>> {
             }
         })
         .flat_map(|args| args.split_first().map(|(t, a)| (t.clone(), a.to_vec())))
-        .map(|(t, args)| {
-            // can be any of the following:
-            // - ./path/to/script
-            // - ~/path/to/script
-            // - /path/to/script
-            // - ../path/to/script
-            // - C:\path\to\script
-            // - .\path\to\script
-            if regex!(r#"^((\.*|~)(/|\\)|\w:\\)"#).is_match(&t) {
-                let path = PathBuf::from(&t);
-                if path.exists() {
-                    let config_root = Config::get()
-                        .project_root
-                        .clone()
-                        .or_else(|| dirs::CWD.clone())
-                        .unwrap_or_default();
-                    let task = Task::from_path(&path, &PathBuf::new(), &config_root)?;
-                    return Ok(vec![task.with_args(args)]);
-                }
+        .collect::<Vec<_>>();
+    let mut tasks = vec![];
+    let arg_re = regex!(r#"^((\.*|~)(/|\\)|\w:\\)"#);
+    for (t, args) in args {
+        // can be any of the following:
+        // - ./path/to/script
+        // - ~/path/to/script
+        // - /path/to/script
+        // - ../path/to/script
+        // - C:\path\to\script
+        // - .\path\to\script
+        if arg_re.is_match(&t) {
+            let path = PathBuf::from(&t);
+            if path.exists() {
+                let config_root = config
+                    .project_root
+                    .clone()
+                    .or_else(|| dirs::CWD.clone())
+                    .unwrap_or_default();
+                let task = Task::from_path(config, &path, &PathBuf::new(), &config_root).await?;
+                return Ok(vec![task.with_args(args)]);
             }
-            let config = Config::get();
-            let tasks = config
-                .tasks_with_aliases()?
-                .get_matching(&t)?
+        }
+        let cur_tasks = config
+            .tasks_with_aliases()
+            .await?
+            .get_matching(&t)?
+            .into_iter()
+            .cloned()
+            .collect_vec();
+        if cur_tasks.is_empty() {
+            if t != "default" || !prompt || !console::user_attended_stderr() {
+                err_no_task(config, &t).await?;
+            }
+            tasks.push(prompt_for_task().await?);
+        } else {
+            cur_tasks
                 .into_iter()
-                .cloned()
-                .collect_vec();
-            if tasks.is_empty() {
-                if t != "default" || !prompt || !console::user_attended_stderr() {
-                    err_no_task(&t)?;
-                }
+                .map(|t| t.clone().with_args(args.to_vec()))
+                .for_each(|t| tasks.push(t));
+        }
+    }
+    Ok(tasks)
+}
 
-                Ok(vec![prompt_for_task()?])
-            } else {
-                Ok(tasks
-                    .into_iter()
-                    .map(|t| t.clone().with_args(args.to_vec()))
-                    .collect())
-            }
+pub async fn resolve_depends(tasks: Vec<Task>) -> Result<Vec<Task>> {
+    let config = Config::get().await;
+    let all_tasks = config.tasks_with_aliases().await?;
+    tasks
+        .into_iter()
+        .map(|t| {
+            let depends = t.all_depends(&all_tasks)?;
+            Ok(once(t).chain(depends).collect::<Vec<_>>())
         })
         .flatten_ok()
         .collect()
