@@ -1,6 +1,6 @@
 use std::path::{Path, PathBuf};
 
-use once_cell::sync::Lazy;
+use std::sync::LazyLock as Lazy;
 
 use crate::env;
 
