@@ -10,6 +10,7 @@ use std::iter::once;
 use std::path::{Path, PathBuf};
 use std::sync::LazyLock as Lazy;
 use std::sync::Mutex;
+use tokio::sync::OnceCell;
 
 #[derive(
     Debug,
@@ -47,31 +48,40 @@ pub fn schedule_hook(hook: Hooks) {
     mu.insert(hook);
 }
 
-pub fn run_all_hooks(ts: &Toolset, shell: &dyn Shell) {
-    let mut mu = SCHEDULED_HOOKS.lock().unwrap();
-    for hook in mu.drain(..) {
-        run_one_hook(ts, hook, Some(shell));
+pub async fn run_all_hooks(ts: &Toolset, shell: &dyn Shell) {
+    let hooks = {
+        let mut mu = SCHEDULED_HOOKS.lock().unwrap();
+        mu.drain(..).collect::<Vec<_>>()
+    };
+    for hook in hooks {
+        run_one_hook(ts, hook, Some(shell)).await;
     }
 }
 
-static ALL_HOOKS: Lazy<Vec<(PathBuf, Hook)>> = Lazy::new(|| {
-    let config = Config::get();
-    let mut hooks = config.hooks().cloned().unwrap_or_default();
-    let cur_configs = config.config_files.keys().cloned().collect::<IndexSet<_>>();
-    let prev_configs = &hook_env::PREV_SESSION.loaded_configs;
-    let old_configs = prev_configs.difference(&cur_configs);
-    for p in old_configs {
-        if let Ok(cf) = config_file::parse(p) {
-            if let Ok(h) = cf.hooks() {
-                hooks.extend(h.into_iter().map(|h| (cf.config_root(), h)));
+async fn all_hooks() -> &'static Vec<(PathBuf, Hook)> {
+    static ALL_HOOKS: OnceCell<Vec<(PathBuf, Hook)>> = OnceCell::const_new();
+    ALL_HOOKS
+        .get_or_init(async || {
+            let config = Config::get().await;
+            let mut hooks = config.hooks().await.cloned().unwrap_or_default();
+            let cur_configs = config.config_files.keys().cloned().collect::<IndexSet<_>>();
+            let prev_configs = &hook_env::PREV_SESSION.loaded_configs;
+            let old_configs = prev_configs.difference(&cur_configs);
+            for p in old_configs {
+                if let Ok(cf) = config_file::parse(p) {
+                    if let Ok(h) = cf.hooks() {
+                        hooks.extend(h.into_iter().map(|h| (cf.config_root(), h)));
+                    }
+                }
             }
-        }
-    }
-    hooks
-});
+            hooks
+        })
+        .await
+}
 
-pub fn run_one_hook(ts: &Toolset, hook: Hooks, shell: Option<&dyn Shell>) {
-    for (root, h) in &*ALL_HOOKS {
+#[async_backtrace::framed]
+pub async fn run_one_hook(ts: &Toolset, hook: Hooks, shell: Option<&dyn Shell>) {
+    for (root, h) in all_hooks().await {
         if hook != h.hook || (h.shell.is_some() && h.shell != shell.map(|s| s.to_string())) {
             continue;
         }
@@ -102,7 +112,7 @@ pub fn run_one_hook(ts: &Toolset, hook: Hooks, shell: Option<&dyn Shell>) {
         }
         if h.shell.is_some() {
             println!("{}", h.script);
-        } else if let Err(e) = execute(ts, root, h) {
+        } else if let Err(e) = execute(ts, root, h).await {
             warn!("error executing hook: {e}");
         }
     }
@@ -145,7 +155,7 @@ impl Hook {
     }
 }
 
-fn execute(ts: &Toolset, root: &Path, hook: &Hook) -> Result<()> {
+async fn execute(ts: &Toolset, root: &Path, hook: &Hook) -> Result<()> {
     SETTINGS.ensure_experimental("hooks")?;
     let shell = SETTINGS.default_inline_shell()?;
 
@@ -155,8 +165,8 @@ fn execute(ts: &Toolset, root: &Path, hook: &Hook) -> Result<()> {
         .map(|s| s.as_str())
         .chain(once(hook.script.as_str()))
         .collect_vec();
-    let config = Config::get();
-    let mut env = ts.full_env(&config)?;
+    let config = Config::get().await;
+    let mut env = ts.full_env(&config).await?;
     if let Some(cwd) = dirs::CWD.as_ref() {
         env.insert(
             "MISE_ORIGINAL_CWD".to_string(),
