@@ -86,12 +86,14 @@ impl FromStr for PathEnv {
 #[cfg(test)]
 mod tests {
     use pretty_assertions::assert_eq;
-    use test_log::test;
+
+    use crate::config::Config;
 
     use super::*;
 
-    #[test]
-    fn test_path_env() {
+    #[tokio::test]
+    async fn test_path_env() {
+        let _config = Config::get().await;
         let mut path_env = PathEnv::from_iter(
             [
                 "/before-1",
@@ -113,8 +115,9 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_path_env_no_mise() {
+    #[tokio::test]
+    async fn test_path_env_no_mise() {
+        let _config = Config::get().await;
         let mut path_env = PathEnv::from_iter(
             [
                 "/before-1",
@@ -134,8 +137,9 @@ mod tests {
             format!("/1:/2:/3:/before-1:/before-2:/before-3:/after-1:/after-2:/after-3")
         );
     }
-    #[test]
-    fn test_path_env_with_colon() {
+    #[tokio::test]
+    async fn test_path_env_with_colon() {
+        let _config = Config::get().await;
         let mut path_env = PathEnv::from_iter(["/item1", "/item2"].map(PathBuf::from));
         path_env.add("/1:/2".into());
         assert_eq!(path_env.to_string(), format!("/1:/2:/item1:/item2"));
